# calibrestekje

[![Repository license](https://img.shields.io/badge/license-GPL-brightgreen.svg)](LICENSE)
[![PyPI package](https://badge.fury.io/py/calibrestekje.svg)](https://badge.fury.io/py/calibrestekje)
[![Travis CI result](https://travis-ci.com/decentral1se/calibrestekje.svg?branch=master)](https://travis-ci.com/decentral1se/calibrestekje)
[![Documentation status](https://readthedocs.org/projects/calibrestekje/badge/?version=latest)](https://calibrestekje.readthedocs.io/en/latest/)
[![Support badge](http://img.shields.io/liberapay/patrons/decentral1se.svg?logo=liberapay)](https://liberapay.com/decentral1se)

## Library prototyping based on Calibre {#introduction}

Calibrestekje is a Python library which provides a way to work with the
[Calibre](https://calibre-ebook.com/) database outside the context of
the Calibre desktop and web interfaces.

Generated [SQLAlchemy](https://docs.sqlalchemy.org/) database bindings
(see [sqlacodegen](https://github.com/agronholm/sqlacodegen) for more)
are provided which allow for read/write access to an existing Calibre
database. These bindings are more fine grained than Calibres [database
interface](https://manual.calibre-ebook.com/db_api.html) and provide
direct access to the Database table layer. The bindings are generated
from Calibre version `2.75.1`.

A [flask](https://flask.palletsprojects.com/) extension is also provided
for getting started with web prototyping. Please see
[flask-calibrestekje](https://github.com/decentral1se/flask-calibrestekje)
and [the flask usage
documentation](https://calibrestekje.readthedocs.io/en/latest/flask.html)
for more.

### Quick Example {#example}

``` python
from calibrestekje import Book, Publisher, init_session

session = init_session("sqlite:///mymetadata.db")

publisher = (session.query(Publisher)
                    .filter(Publisher.name == "MIT Press").one())

books = (session.query(Book)
                .filter(Book.publishers.contains(publisher)))

print(f"Books published by MIT Press: {books.count()}")
```

### Documentation

-   <https://calibrestekje.readthedocs.io/>
